﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace TextDetection.Controls
{
    public class PictureBoxControl
    {
        public Bitmap Bitmap { get => (Bitmap)_pictureBox.Image; }
        private readonly PictureBox _pictureBox;
        private readonly List<Rectangle> _textAreas;

        public PictureBoxControl(PictureBox pictureBox)
        {
            _pictureBox = pictureBox;
            _textAreas = new List<Rectangle>();
            _pictureBox.Paint += _pictureBox_Paint;
        }

        private void _pictureBox_Paint(object sender, PaintEventArgs e)
        {
            foreach (var textAreaRect in _textAreas)
                e.Graphics.DrawRectangle(new Pen(new SolidBrush(Color.Red), 2), textAreaRect);
        }

        public void OpenImage(string imageFilePath, Action imageLoaded)
        {
            _pictureBox.Image = Image.FromFile(imageFilePath);
            imageLoaded();
        }

        public void DrawRectangle(Rectangle rectangle)
        {
            _textAreas.Add(rectangle);
            _pictureBox.Invalidate();
        }

        public void SetImage(Bitmap bitmap)
        {
            _pictureBox.Image = bitmap;
        }
    }
}
